<?php
/**
 * Availability widget class file.
 * 
 * @author Stefano Chiodino ChiodinoStefano@gmail.com
 * @copyright 2011
 * @version $0.9
 * @package Yii Framework < http://yiiframework.com >
 * @subpackage Widgets
 * 
 * This widget require an "availability" column like:
 * availabiliy TEXT NULL DEFAULT NULL
 */
class Availability extends CWidget
{
    /**
     * @var boolean - if the table should be modified or displayed only
     */
	public $readOnly = true;
    /**
     * @var string - the user id, if unset will be the current logged user
     */
	public $userId;

	/**
	 * Check for availability column in user table, unserialize the data
	 * and builds up css selector for the cells to be selected.
	 * - If in readOnly mode it renders the table.
     * - If not in readOnly mode it renders a table within a form, with 
     * selectable cells and hidden checkboxes inside. On submit the script
     * check the boxes inside selected cells, then the widget serialize
     * the array and store it into the DB
	 */
	public function init()
	{
		if( isset( $this->userId ) ) {
			if( $this->userId == Yii::app()->user->getId() )
				$this->readOnly = false;
		}
		else {
            if(Yii::app()->user->isGuest)
                return;
			$this->userId = Yii::app()->user->getId();
		}
		$userRecord = User::model()->findByPk( $this->userId );
		if( isset( $userRecord->availability ) ) {
			$availability = unserialize( $userRecord->availability );

			$selectedCells = array();
			$index = 0;
			foreach( Date::getPeriods() as $period )
				foreach( Date::getDays() as $day )
					if( array_key_exists( $index++, $availability ) )
						$selectedCells[] = 'tr.' . $period . ' .' . $day;

			Yii::app()->clientScript->registerScript( 'availabilityInitScript',
                "$(document).ready(function(){
                    $('" . implode( $selectedCells, ', ' ) . "').addClass('selected');
                });" );
			if( $this->readOnly )
				$this->render( 'availability' );
		}
		if( ! $this->readOnly ) {
			if( isset( $_POST['availability-form'] ) ) {
				$userRecord->availability = serialize( $_POST['availability-form'] );
				if( $userRecord->save() )
					Yii::app()->user->setFlash( 'success', 'Disponibilit&agrave; modificata!' );
				else
					Yii::app()->user->setFlash( 'error', 'Errore nella modifica della disponibilit&agrave;!' );
				$this->controller->refresh();
			}
			Yii::app()->clientScript->registerScript( 'availabilityScript',
                "$(document).ready(function(){
                    // table cells
            		$( '#availability td' ).click(function() {
                        $(this).toggleClass('selected');
                    });
                    // table row headers
            		$( '#availability tbody th' ).click(function() {
                        $(this).parent().children().toggleClass('selected');
                    });
                    // table column headers
            		$( '#availability thead th' ).click(function() {
                        $('.'+$(this).attr('class').replace(' selected', '')).toggleClass('selected');
                    });
                    // table select all
            		$( '#availability .selectAll' ).click(function() {
                        $('#availability td').toggleClass('selected');
                    });
                    // on submit check all boxes within selected class td
                    $('#availability-form').submit(function() {
                        $('.selected input').attr('checked','checked');
                    });
                });" );
			$this->render( 'availabilityEdit' );
		}
	}
}
